/*
*==============================================================================
*
*  Name   :  RMSObjsFuncs.js
*
*  Version:  1.0
*
*  Purpose:  Fast search and auxiliary functions for RAMSES objects identifiers
*            using pre-sorted identifiers lists.
*
*  Remarks:  Derived from initial work by Dobrica Pavlinusic (dpavlin@rot13.org 2004-09-06)
*            and Matko Andjelinic (matko.andjelinic@gmail.com 2004-09-09).
*            See http://docman.sourceforge.net/home_html/projects/bfilter/README.html     
*
*==============================================================================
* Implementation and Revisions:
*------------------------------------------------------------------------------
* Author  Date      Description
* ------  ----      -----------
* dg      27/07/06  First implementation 
* dg      31/07/06  Replaced list output by table output   
* dg      07/08/06  Search made case sensitive
*                   - Now supporting global flag searchWord   
*                   - Restructured, simplified, identifiers renamed   
* dg      08/08/06  New: DoSearch, DoDelayedSearch, DoImmediateSearch
* dg      09/08/06  Now supporting regular expression search
* dg      10/08/06  Now always doing delayed search in order to allow for 
*                   window update (making ShowSrchMsg effective)   
* dg      13/08/06  GetDEFLink, GetQRLink: special treatment if 
*                   object = module (theObjNo == 0)
* dg      14/08/06  ShowSearchResults: table layout redefined (now table <> 100%)
* dg      18/08/06  FindHtmlSectionObject adapted to handle Netscape 4.x and 
*                   other browsers not supporting "document.getElementById"
* af      19/08/10  'top._rmsSearchObj' replaced by 'document._rmsSearchObj'
*                   to make search function also functional from within 
*                   SILVA's iframe technique
*
*==============================================================================
*/


document._rmsSearchObj = new Array();


function RMSSearch( index ) {
  // Constructor for "RMSSearch" object
  if (! index) { this.ShowWarning( "index not defined or empty" ); return };
  if (! index.partIdLength) { this.ShowWarning( "index.partIdLength not defined or equal 0" ); return };
  if (index.partIdLength < 0) { this.ShowWarning( "index.partIdLength must be > 0" ); return };
  this.htmlSectionObjectsArr = Array();
  this.index = index;
  this.partIdLength = index.partIdLength;
  this.ShowDebugInfo("partIdLength = " + this.partIdLength );
  this._objCount = document._rmsSearchObj.length; // store reference to this object for later
  document._rmsSearchObj[this._objCount] = this;
  this.timeoutHandle = null;
  this.timeOutDuration = 200; // true delay: show results after 0.20s
  this.timeOutDuration = 5;   // small delay (give time for window update)
  this.resultsList = null;    // clear results list
  this.numFound = 0;          // total number of entries found
  this.lastFoundEntry = "";   // the last found entry
  this.atSearchEndFunc = function() { return 0 }; // function to call at end of search
}


RMSSearch.prototype.EmptyAtSearchEndFunc = function () {
  return 0;
}
    

RMSSearch.prototype.ClearResults = function () {
  var s = this.FindHtmlSectionObject("searchResultsSection");
  s.innerHTML = "";
  this.resultsList = "";
  this.numFound = 0;
  this.lastFoundEntry = "";
  return true;
}


RMSSearch.prototype.FindHtmlSectionObject = function (sectionId, no_alert) {
  if (this.htmlSectionObjectsArr[sectionId]) { return this.htmlSectionObjectsArr[sectionId] };
  if (document.getElementById) {
    var el = self.document.getElementById(sectionId);
    if (el) { this.htmlSectionObjectsArr[sectionId] = el; return el };
    this.ShowWarning( "Can't find section: '" + sectionId + "'");
    this.htmlSectionObjectsArr[sectionId] = null;
    return true;
  } else if (document.layers) {
    alert("Sorry, 'RAMSES Objects Search' does not function with Netscape 4.x");
  } else {
    alert("Sorry, 'RAMSES Objects Search' does not function with this browser");
  }
}


RMSSearch.prototype.ShowWarning = function ( warning ) {
  alert( "RMSObjsFuncs.js" + "\n" + warning );
}


RMSSearch.prototype.ShowDebugInfo = function( htmlText ) {
return null;
  var s = this.FindHtmlSectionObject("debugSection",1);
  if (! htmlText) { return 1 }
  s.innerHTML += "<br>" + htmlText;
  return true;
}


RMSSearch.prototype.ShowSrchMsg = function () {
  var s = this.FindHtmlSectionObject("statusSection");
  var htmlText = "<table style='font-size: 12; color: gray' cellpadding='0' cellspacing='0' border='0' width=100%>"; 
  htmlText = htmlText + "<tr><td valign='top'>";
  htmlText = htmlText + "&nbsp;&nbsp;&nbsp;&nbsp;";
  htmlText = htmlText + "Searching...";
  htmlText = htmlText + "</td></tr>";
  htmlText = htmlText + "</table>";
  s.innerHTML = htmlText;
  return true;
}


RMSSearch.prototype.GetRegExpHelpText = function () {
  var text = "" ;
  text = text + "              </td><td>[...]  </td><td>Any one character between the brackets.</td></tr>" ;
  text = text + "<tr><td>&nbsp;</td><td>[^...] </td><td>Any one character not between the brackets.</td></tr>" ;
  text = text + "<tr><td>&nbsp;</td><td>.      </td><td>Any character.</td></tr>" ;
  text = text + "<tr><td>&nbsp;</td><td>\\w    </td><td>Any word character. Equivalent to [a-zA-Z0-9_].</td></tr>" ;
  text = text + "<tr><td>&nbsp;</td><td>\\s    </td><td>Any digit. Equivalent to [0-9].</td></tr>" ;
  text = text + "<tr><td>&nbsp;</td><td>?      </td><td>Match zero or one occurrences of the previous item.</td></tr>" ;
  text = text + "<tr><td>&nbsp;</td><td>+      </td><td>Match one or more occurrences of the previous item.</td></tr>" ;
  text = text + "<tr><td>&nbsp;</td><td>*      </td><td>Match zero or more occurrences of the previous item." ;
  return text
}
        
RMSSearch.prototype.ShowStatus = function (searchString, searchType) {
  var s = this.FindHtmlSectionObject("statusSection");
  var htmlText = "<table style='font-size: 12; color: gray' cellpadding='0' cellspacing='0' border='0' width=100%>"; 
  htmlText = htmlText + "<tr><td valign='top'>";
  htmlText = htmlText + "&nbsp;&nbsp;&nbsp;&nbsp;";
  if (this.numFound == 0)        { if (searchString!="") {htmlText = htmlText + "No match.&nbsp;&nbsp;"}
  } else if (this.numFound == 1) { htmlText = htmlText + "<b>1</b> match.&nbsp;&nbsp;";
  } else                         { htmlText = htmlText + "<b>" + this.numFound + "</b> matches.&nbsp;&nbsp;";
  }
  switch ( searchType ) {
    case 0: ;
      htmlText = htmlText + "Enter object identifier.";
      break;
    case 1: 
      if (! searchString) {
        htmlText = htmlText + "Enter a letter to filter object identifiers.";
      } else {
        if (this.numFound != 0)        {
          htmlText = htmlText + "Enter a letter to filter object identifiers <i><b>" + searchString + "</b></i>.*.";
        }
      }
      break;
    case 2: 
      htmlText = htmlText + "Enter regular expression."; 
      htmlText = htmlText + this.GetRegExpHelpText();
      break;
    default: 
      ;
  }
  htmlText = htmlText + "</td></tr>";
  htmlText = htmlText + "</table>";
  s.innerHTML = htmlText;
  return true;
}


RMSSearch.prototype.NumFoundEntries = function () {
  return this.numFound;
}


RMSSearch.prototype.LastFoundEntry = function () {
  return this.lastFoundEntry;
}


RMSSearch.prototype.GetDEFLink = function (indexEntry) {
  var theObjNo   = indexEntry[1];
  var theModNo   = indexEntry[3];
  var theModId   = modId    [ indexEntry[3] ];
  var theSWLayer = swLayerId[ indexEntry[4] ];
	if (theObjNo == 0) {
    return theSWLayer + "/" + theModId + ".html#" + theModId ;
  } else {
    return theSWLayer + "/" + theModId + ".html#" + theModNo + "_" + theObjNo ;
  }
}


RMSSearch.prototype.GetQRLink = function (indexEntry) {
  var theObjId   = indexEntry[0];
  var theObjNo   = indexEntry[1];
  var theModNo   = indexEntry[3];
  var theSWLayer = swLayerId[ indexEntry[4] ];
	if (theObjNo == 0) {
    return theSWLayer + "/" + theSWLayer + "_Quick_Reference.html" + "#" + theObjId ;
  } else {
    return theSWLayer + "/" + theSWLayer + "_Quick_Reference.html" + "#" + theModNo + "_" + theObjNo ;
  }
}


RMSSearch.prototype.DocumentResult = function (i, indexEntry) {
  var theObjId   = indexEntry[0];
  var theObjType = objTypeId[ indexEntry[2] ];
  var theModId   = modId    [ indexEntry[3] ];
  var theSWLayer = swLayerId[ indexEntry[4] ];
  var link1      = theObjId.link  ( this.GetDEFLink( indexEntry ) );
  var link2      = theSWLayer.link( this.GetQRLink ( indexEntry ) );
  this.lastFoundEntry = indexEntry;
  this.resultsList += 
      "<tr>" +
      "<td align='center' valign='center'><i>" + i + "</i></td>" +
      "<td>" + link1 + "&nbsp;</td>" +
      "<td align='center'>" + theObjType + "</td>" +
      "<td>" + theModId + "&nbsp;&nbsp;</td>" +
      "<td>" + link2 + "&nbsp;&nbsp;</td>" +
      "</tr>";
  return true;
}


RMSSearch.prototype.ShowSearchResults = function () {
  var s = this.FindHtmlSectionObject("searchResultsSection");
  if (this.resultsList) {
    s.innerHTML = 
      "<br><table style='font-size: 12' cellpadding='0' cellspacing='0' border='0'>" +
      "<tr>" + 
      "<td width=40 align='center'><i>No<br></i></td>" +
      "<td width=200><i>Object</i></td>" +
      "<td width=50 align='center'><i>Type</i></td>" +
      "<td width=100><i>Module</i></td>" +
      "<td width=50><i>QuickRef</i></td>" +
      "</tr>" +
      this.resultsList +
      "<table>";
  }
  return true;
}


RMSSearch.prototype.BinarySearch = function (index, searchString) {
  // Modified binary search to find first element with substring
  if (!index || typeof (searchString) == "undefined" || !index.length) {
    return null;
  }
  var low = 0;
  var high = index.length - 1;
  var middlearr = parseInt(index.length / 2);
  this.ShowDebugInfo("BinarySearch: "+low+"-("+middlearr+")-"+high+" for "+searchString);
  var lastTry;
  while (low <= high) {
    var mid = (low + high) / 2;
    var aTry = (mid < 1) ? 0 : parseInt(mid);
    var curr = index[aTry][0].substr(0,searchString.length);
    this.ShowDebugInfo(low+"-"+high+", "+aTry+"="+curr+" last="+lastTry);
    if (curr < searchString) {
      low = aTry + 1;
      continue;
    }
    if (curr > searchString) {
      high = aTry - 1;
      continue;
    }
    if (curr == searchString) {
      high = aTry - 1;
      lastTry = aTry;
      continue;
    }
    return aTry;
  }
  this.ShowDebugInfo("lastTry="+lastTry);
  if (typeof (lastTry) != "undefined") {
    return lastTry;
  } else {
    return null;
  }
}


RMSSearch.prototype.DoSearch = function (searchString, searchType) {
  this.ShowSrchMsg();
  this.DoDelayedSearch( searchString, searchType )
}
    
    
RMSSearch.prototype.DoDelayedSearch = function (searchString, searchType) {
  if (this.timeoutHandle) {
    clearTimeout(this.timeoutHandle);
    this.timeoutHandle = null;
  }
  this.timeoutHandle = 
    setTimeout("document._rmsSearchObj[" + this._objCount + "]." +
               "DoImmediateSearch('" + searchString.replace(/'/,"\\'") + "', " + searchType + ");", 
               this.timeOutDuration );
  return true;
}


RMSSearch.prototype.DoImmediateSearch = function (searchString, searchType) {
  // searchType: 0 = "exact match", 1 = "starts with", 2 = "regular expression"
  this.ClearResults();
  if (this.timeoutHandle) {
    clearTimeout(this.timeoutHandle);
    this.timeoutHandle = null;
  }
  switch ( searchType ) {
    case 0: ;
    case 1: 
      if (searchString.length < this.partIdLength) { this.ShowStatus("", searchType); return }
      var part = searchString.substr(0,this.partIdLength);
      if (! this.index[part]) { this.ShowStatus(searchString, searchType); return }
      if (searchString.length == this.partIdLength) {
        if (searchType == 0) {
          if (this.index[part][0][0] == part) {
            this.numFound++; this.DocumentResult(this.numFound, this.index[part][0]);
          }
        } else {
          for (var i = 0; i < this.index[part].length; i++) {
            this.numFound++; this.DocumentResult(this.numFound, this.index[part][i]);
          }
        }
      } else {
        var frstFound = this.BinarySearch(this.index[part], searchString);
        if (frstFound != null) {
          if (searchType == 0) { 
             for(var i = frstFound ; i < this.index[part].length ; i++) {
               if (this.index[part][i][0] != searchString) { break };
               this.numFound++; this.DocumentResult(this.numFound, this.index[part][i]);
             }
           } else {
             for(var i = frstFound ; i < this.index[part].length ; i++) {
               if (this.index[part][i][0].substring(0,searchString.length) != searchString) { break };
               this.numFound++; this.DocumentResult(this.numFound, this.index[part][i]);
             }
           }
         }
      }
      break;
    case 2: 
      //this.ShowWarning( "searching" );
      var re = new RegExp("^"+searchString+"$", "");
      for (var part in this.index) {
        for (var i = 0; i < this.index[part].length; i++) {
          if (re.exec(this.index[part][i][0]))  {
            this.numFound++; this.DocumentResult(this.numFound, this.index[part][i]);
          }
        }
      }
      break;
    default: 
      this.ShowWarning( "Unexpected value for searchType: '" + searchType + "'" );
  }
  this.ShowSearchResults();
  this.ShowStatus(searchString, searchType);
  this.atSearchEndFunc();
}

